package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamStudentCourseDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.pdf.ExamStudentInfo;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.mapper.ExamStudentMapper;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.ExamStudentService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamStudentServiceImpl extends ServiceImpl<ExamStudentMapper, ExamStudent> implements ExamStudentService {

    @Resource
    private ExamStudentMapper examStudentMapper;
    @Resource
    private ExamDetailCourseMapper examDetailCourseMapper;
    @Resource
    private BasicExamStudentService basicExamStudentService;

    @Override
    public List<String> listPaperTypeByExamDetailCourseId(String examDetailCourseId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCourseId);
        List<ExamStudent> examStudents = this.list(queryWrapper);
        if (!examStudents.isEmpty()) {
            return examStudents.stream().filter(m -> StringUtils.isNotBlank(m.getPaperType())).map(m -> m.getPaperType()).distinct().collect(Collectors.toList());
        } else {
            ExamDetailCourse examDetailCourse = examDetailCourseMapper.selectById(examDetailCourseId);
            if (examDetailCourse != null && StringUtils.isNotBlank(examDetailCourse.getPaperType())) {
                return Arrays.stream(examDetailCourse.getPaperType().split(",")).collect(Collectors.toList());
            }
        }
        return null;
    }

    @Override
    public Map<String, Object> getStudentDetail(Long id) {
        Map<String, Object> map = this.baseMapper.getStudentDetail(id);
        Object extendFields = map.get("extendFields");
        if (Objects.nonNull(extendFields)) {
            List<Map> list = JSONObject.parseArray(extendFields.toString(), Map.class);
            map.put("extendFields", list);
        }
        return map;
    }

    /**
     * 根据科目查询考生
     *
     * @param schoolId
     * @param examDetailCourseIds
     * @return
     */
    @Override
    public List<ExamStudentCourseDto> queryBySchoolIdAndExamDetailCourseIds(Long schoolId, List<Long> examDetailCourseIds) {
        return examStudentMapper.queryBySchoolIdAndExamDetailCourseIds(schoolId, examDetailCourseIds);
    }

    @Override
    public void removeByExamDetailCourseId(Long id) {
        UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, id);
        this.remove(updateWrapper);
    }

    @Override
    public List<ExamStudent> listByExamDetailCourseIdNotIntMarkStudent(Long examDetailCourseId, String paperType) {
        return this.baseMapper.listByExamDetailCourseIdNotIntMarkStudent(examDetailCourseId, paperType);
    }

    @Override
    public List<ExamStudent> listByExamDetailCourseId1(Long examDetailCourseId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCourseId)
                .orderByAsc(ExamStudent::getId);
        return this.list(queryWrapper);
    }

    /**
     * 查询班级和老师信息
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    public ExamStudentDto queryExamStudent(Long examId, String courseCode, String paperNumber) {
        return this.baseMapper.queryExamStudent(examId, courseCode, paperNumber);
    }

    @Override
    public List<ExamStudent> saveStudentForExamApply(ExamTask examTask, String extendFields, List<Long> examTaskStudentObjectParamList, Long examDetailCourseId, SysUser sysUser) {
        List<ExamStudent> examStudentList = new ArrayList<>();
        for (Long studentId : examTaskStudentObjectParamList) {
            BasicExamStudent basicExamStudent = basicExamStudentService.getById(studentId);
            if (basicExamStudent == null) {
                continue;
            }
            ExamStudent examStudent = new ExamStudent();
            examStudent.setId(SystemConstant.getDbUuid());
            examStudent.setSchoolId(sysUser.getSchoolId());
            examStudent.setExamDetailCourseId(examDetailCourseId);
            examStudent.setBasicStudentId(studentId);
            examStudent.setStudentCode(basicExamStudent.getStudentCode());
            examStudent.setExamId(examTask.getExamId());
            examStudent.setPaperNumber(examTask.getPaperNumber());
            examStudent.setCoursePaperId(String.valueOf(examTask.getId()));
            examStudent.setExtendFields(extendFields);
            examStudentList.add(examStudent);
        }
        return examStudentList;
    }

    @Override
    public List<ExamStudentInfo> listByExamDetailCourseId(Long examDetailCourseId) {
        return this.baseMapper.listByExamDetailCourseId(examDetailCourseId);
    }

    @Override
    public void updateAttachmentIdById(ExamStudentInfo t) {
        UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamStudent::getAttachmentId, t.getAttachmentId()).eq(ExamStudent::getId, t.getExamStudentId());
        this.update(updateWrapper);
    }
}
