package com.qmth.teachcloud.obe.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.enums.SourceEnum;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.mark.bean.archivescore.MarkStudentScoreVo;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.obe.been.dto.FinalScoreDto;
import com.qmth.teachcloud.obe.been.dto.TCFinalScoreDto;
import com.qmth.teachcloud.obe.been.excel.PaperStructDto;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.been.result.ObeScoreResult;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.entity.TCFinalScore;
import com.qmth.teachcloud.obe.entity.TCPaperStruct;
import com.qmth.teachcloud.obe.entity.TCUsualScore;
import com.qmth.teachcloud.obe.mapper.TCFinalScoreMapper;
import com.qmth.teachcloud.obe.mapper.TCUsualScoreMapper;
import com.qmth.teachcloud.obe.service.TCFinalScoreService;
import com.qmth.teachcloud.obe.service.TCPaperStructService;
import com.qmth.teachcloud.obe.service.TCUsualScoreService;
import com.qmth.teachcloud.obe.service.TRBasicInfoService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 期末考试成绩表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCFinalScoreServiceImpl extends ServiceImpl<TCFinalScoreMapper, TCFinalScore> implements TCFinalScoreService {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    MarkStudentService markStudentService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    TCUsualScoreService tcUsualScoreService;

    @Resource
    private TCUsualScoreMapper tcUsualScoreMapper;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    /**
     * 导入期末成绩excel
     *
     * @param file
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> finalScoreExcelImport(MultipartFile file, Long examId, Long courseId, String paperNumber, Long cultureProgramId) {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            printCommonService.scoreImportExcelVaild(file, examId, null, paperNumber, courseId);
            Double totalScore = 100d;

            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<String[]> arrayList = excelReader.getDataArrayList();
            log.info("arrayList:{}", JacksonUtil.parseJson(arrayList));

            if (!CollectionUtils.isEmpty(arrayList)) {
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                List<TCFinalScore> tcFinalScoreList = new ArrayList<>(arrayList.size());
                String[] columnNames = excelReader.getColumnNames();
                for (int i = 0; i < columnNames.length; i++) {
                    if (i == 0 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "学号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                    } else if (i == 1 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "姓名")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                    } else if (i == 2 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "成绩")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第三行为成绩");
                    }
                }

                BasicCourse basicCourse = basicCourseService.getById(courseId);
                if (Objects.isNull(basicCourse)) {
                    throw ExceptionResultEnum.ERROR.exception("课程不存在");
                }
                TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber);
                Objects.requireNonNull(tcPaperStruct, "未找到试卷结构");

                List<PaperStructDimensionResult> paperStructDimensionResultList = null;
                if (Objects.nonNull(tcPaperStruct.getPaperStructDimension())) {
                    paperStructDimensionResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStructDimension(), new TypeToken<List<PaperStructDimensionResult>>() {
                    }.getType());
                } else {
                    List<PaperStructDto> paperStructDtoList = GsonUtil.fromJson(tcPaperStruct.getPaperStruct(), new TypeToken<List<PaperStructDto>>() {
                    }.getType());
                    paperStructDimensionResultList = new ArrayList<>(paperStructDtoList.size());
                    for (PaperStructDto paperStructDto : paperStructDtoList) {
                        paperStructDimensionResultList.add(new PaperStructDimensionResult(paperStructDto.getMainNumber(), paperStructDto.getSubNumber(), tcPaperStruct.getExamId(), tcPaperStruct.getPaperNumber(), new BigDecimal(paperStructDto.getScore())));
                    }
                }
                Map<String, PaperStructDimensionResult> paperStructResultMap = paperStructDimensionResultList.stream().collect(Collectors.toMap(k -> k.getMainNumber() + "-" + k.getSubNumber(), Function.identity(), (dto1, dto2) -> dto1));

                for (int i = 0; i < arrayList.size(); i++) {
                    String[] strs = arrayList.get(i);
                    if (Objects.nonNull(strs) && strs.length > 0) {
                        if (strs.length != columnNames.length) {
                            throw ExceptionResultEnum.ERROR.exception("excel列和数据不匹配");
                        }
                        boolean error = false;
                        List<TCFinalScoreDto> tcFinalScoreDtoList = new ArrayList<>();
                        TCFinalScore tcFinalScore = new TCFinalScore(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, SourceEnum.EXCEL_IMPORT, sysUser.getId());
                        BigDecimal score = new BigDecimal(0);
                        for (int j = 0; j < strs.length; j++) {
                            if (Objects.isNull(strs[j]) || Objects.equals(strs[j].trim(), "")) {
                                errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j] + "]为空;").add("\r\n");
                                error = true;
                            } else {
                                if (Objects.equals(columnNames[j].trim(), "学号")) {
                                    tcFinalScore.setStudentCode(strs[j]);
                                } else if (Objects.equals(columnNames[j].trim(), "姓名")) {
                                    tcFinalScore.setName(strs[j]);
                                } else if (Objects.equals(columnNames[j].trim(), "成绩")) {
                                    Double scoreTemp = SystemConstant.isDouble(strs[j]);
                                    if (Objects.nonNull(scoreTemp)) {
                                        tcFinalScore.setScore(scoreTemp);
                                    } else {
                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]请输入数字").add("\r\n");
                                        error = true;
                                    }
                                } else {
                                    if (Objects.nonNull(SystemConstant.isDouble(strs[j]))) {
                                        if (!paperStructResultMap.containsKey(columnNames[j].trim())) {
                                            errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]题号不正确").add("\r\n");
                                            error = true;
                                        } else {
                                            PaperStructDimensionResult paperStructDimensionResult = paperStructResultMap.get(columnNames[j].trim());
                                            BigDecimal paperScore = paperStructDimensionResult.getScore();
                                            BigDecimal importScore = new BigDecimal(strs[j]);
                                            if (importScore.compareTo(paperScore) == 1) {
                                                errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]导入分数大于题号" + columnNames[j] + "的总分").add("\r\n");
                                                error = true;
                                            } else {
                                                score = score.add(importScore);
                                                tcFinalScoreDtoList.add(new TCFinalScoreDto(strs[j], columnNames[j]));
                                            }
                                        }
                                    } else {
                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]请输入数字").add("\r\n");
                                        error = true;
                                    }
                                }
                            }
                        }
                        if (CollectionUtils.isNotEmpty(tcFinalScoreDtoList)) {
                            if (score.doubleValue() > totalScore.doubleValue()) {
                                errorData.add("excel第").add((i + 1) + "").add("行[总分大于试卷总分]").add("\r\n");
                                error = true;
                            } else {
                                score = score.setScale(2, BigDecimal.ROUND_HALF_UP);
                                tcFinalScore.setScore(Double.parseDouble(SystemConstant.df.format(score)));
                                tcFinalScore.setScoreDetail(JacksonUtil.parseJson(tcFinalScoreDtoList));
                            }
                        }
                        if (!error) {
                            tcFinalScoreList.add(tcFinalScore);
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(tcFinalScoreList)) {
                    successData.add("共导入").add(tcFinalScoreList.size() + "").add("条数据");
                    List<TCFinalScore> tcFinalScoreDbSourceList = tcFinalScoreService.queryFinalScore(cultureProgramId, courseId, paperNumber);
                    if (CollectionUtils.isEmpty(tcFinalScoreDbSourceList) || !CollectionUtils.isEqualCollection(tcFinalScoreDbSourceList, tcFinalScoreList)) {
                        tcFinalScoreService.removeByIds(tcFinalScoreDbSourceList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                        tcFinalScoreService.saveBatch(tcFinalScoreList);
                        trBasicInfoService.clearReportData(cultureProgramId, courseId, paperNumber, true);
                    }
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 期末成绩列表（分页）
     *
     * @param iPage
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    @Override
    public IPage<TCFinalScore> finalScoreList(IPage<Map> iPage, Long examId, String courseCode, String paperNumber, Long cultureProgramId, Long courseId) {
        return this.baseMapper.finalScoreList(iPage, cultureProgramId, courseId);
    }

    /**
     * 查找期末成绩分数图
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    @Override
    public FinalScoreDto finalScoreScoreOverView(Long examId, String courseCode, String paperNumber, Long cultureProgramId, Long courseId) {
        return this.baseMapper.finalScoreScoreOverView(cultureProgramId, courseId);
    }

    /**
     * 查找期末成绩分数区间
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param start
     * @param end
     * @return
     */
    @Override
    public int getCountByScoreRange(Long examId, String courseCode, String paperNumber, Long cultureProgramId, Long courseId, Double start, Double end) {
        return this.baseMapper.getCountByScoreRange(cultureProgramId, courseId, start, end);
    }

    /**
     * 查找考生成绩
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    @Override
    public List<FinalScoreResult> examStudentOverview(Long examId, String courseCode, String paperNumber, Long cultureProgramId, Long courseId) {
        return this.baseMapper.examStudentOverview(cultureProgramId, courseId);
    }

    /**
     * 同步期末成绩
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, String> finalScoreSync(Long examId, Long courseId, String paperNumber, Long cultureProgramId) throws IOException {
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");
            List<MarkStudentScoreVo> markStudentScoreVoList = markStudentService.listMarkStudentScoreList(examId, paperNumber);
            if (!CollectionUtils.isEmpty(markStudentScoreVoList)) {
                List<TCFinalScore> tcFinalScoreList = new ArrayList<>(markStudentScoreVoList.size());
                List<PaperStructDimensionResult> paperStructDimensionResultList = new ArrayList<>(markStudentScoreVoList.size());
                for (int i = 0; i < markStudentScoreVoList.size(); i++) {
                    MarkStudentScoreVo markStudentScoreVo = markStudentScoreVoList.get(i);
                    if (!CollectionUtils.isEmpty(markStudentScoreVo.getTotalScoreList())) {
                        List<TCFinalScoreDto> tcFinalScoreDtoList = new ArrayList<>();
                        for (ScoreItem scoreItem : markStudentScoreVo.getTotalScoreList()) {
                            String name = scoreItem.getMainNumber() + "-" + scoreItem.getSubNumber();
                            tcFinalScoreDtoList.add(new TCFinalScoreDto(scoreItem.getScore().toString(), name));
                            if (i == 0) {
                                paperStructDimensionResultList.add(
                                        new PaperStructDimensionResult(scoreItem.getMainNumber(), scoreItem.getSubNumber(), examId, paperNumber,
                                                new BigDecimal(scoreItem.getTotalScore())));
                            }
                        }
                        tcFinalScoreList.add(
                                new TCFinalScore(markStudentScoreVo, JacksonUtil.parseJson(tcFinalScoreDtoList),
                                        SourceEnum.SYNC, sysUser.getId(), cultureProgramId, courseId));
                    }
                }
                if (!CollectionUtils.isEmpty(tcFinalScoreList) && !CollectionUtils.isEmpty(
                        paperStructDimensionResultList)) {
                    successData.add("共同步").add(tcFinalScoreList.size() + "").add("条数据");
                    List<TCFinalScore> tcFinalScoreDbSourceList = tcFinalScoreService.queryFinalScore(cultureProgramId,
                            courseId, null);
                    if (CollectionUtils.isEmpty(tcFinalScoreDbSourceList) || !CollectionUtils.isEqualCollection(
                            tcFinalScoreDbSourceList, tcFinalScoreList)) {
                        tcFinalScoreService.removeByIds(
                                tcFinalScoreDbSourceList.stream().map(BaseEntity::getId).collect(Collectors.toList()));
                        tcFinalScoreService.saveBatch(tcFinalScoreList);
                        trBasicInfoService.clearReportData(cultureProgramId, courseId, paperNumber, true);
                    }
                    TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, null);
                    if (Objects.nonNull(tcPaperStruct)) {
                        tcPaperStructService.removeById(tcPaperStruct.getId());
                    }
                    BasicCourse basicCourse = basicCourseService.getById(courseId);
                    Objects.requireNonNull(basicCourse, "未找到基础课程信息");
                    tcPaperStruct = new TCPaperStruct(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, JacksonUtil.parseJson(paperStructDimensionResultList),
                            sysUser.getId());
                    tcPaperStruct.setExamId(examId);

                    tcPaperStructService.saveOrUpdate(tcPaperStruct);
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return messageMap;
    }

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber) {
        QueryWrapper<TCFinalScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId).eq(TCFinalScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.eq(TCFinalScore::getPaperNumber, paperNumber).or().isNull(TCFinalScore::getPaperNumber));
        }
        return tcFinalScoreService.remove(queryWrapper);
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @return
     */
    @Override
    public List<TCFinalScore> queryFinalScore(Long cultureProgramId, Long courseId, String paperNumber) {
        QueryWrapper<TCFinalScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId).eq(TCFinalScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.eq(TCFinalScore::getPaperNumber, paperNumber).or().isNull(TCFinalScore::getPaperNumber));
        }
        return tcFinalScoreService.list(queryWrapper);
    }

    /**
     * 获取期末考试同步锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     */
    @Override
    public void getFinalScoreSyncLock(Long cultureProgramId, Long courseId, String paperNumber) {
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.SYNC + cultureProgramId + "_" + courseId + "_" + paperNumber;
        Object o = redisUtil.get(lockKey);
        if (Objects.nonNull(o)) {
            throw ExceptionResultEnum.ERROR.exception("正在同步期末成绩数据，请稍候再试！");
        }
    }

    /**
     * 获取期末考试导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     */
    @Override
    public void getFinalScoreImportLock(Long cultureProgramId, Long courseId, String paperNumber) {
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.IMPORT + cultureProgramId + "_" + courseId + "_" + paperNumber;
        Object o = redisUtil.get(lockKey);
        if (Objects.nonNull(o)) {
            throw ExceptionResultEnum.ERROR.exception("正在导入期末成绩数据，请稍候再试！");
        }
    }

    /**
     * 成绩管理列表
     *
     * @param iPage
     * @param cultureProgramId 培养方案id
     * @param semesterId       学期id
     * @param courseId         课程id
     * @return
     */
    @Override
    public IPage<ObeScoreResult> scoreList(IPage<Map> iPage, Long cultureProgramId, Long semesterId, Long courseId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return tcUsualScoreMapper.scoreList(iPage, cultureProgramId, semesterId, courseId, dpr);
    }

    /**
     * 获取成绩管理需要的锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     */
    @Override
    public void getLock(Long cultureProgramId, Long courseId, String paperNumber) {
        tcFinalScoreService.getFinalScoreSyncLock(cultureProgramId, courseId, paperNumber);
        tcFinalScoreService.getFinalScoreImportLock(cultureProgramId, courseId, paperNumber);
        tcPaperStructService.getPaperStructSyncLock(cultureProgramId, courseId, paperNumber);
        tcPaperStructService.getPaperStructImportLock(cultureProgramId, courseId, paperNumber);
    }

    /**
     * 校验方案下课程是否已导入成绩
     *
     * @param cultureProgramId 培养方案ID
     * @param courseId         课程ID
     */
    @Override
    public void checkUsualOrFinalScoreExist(Long cultureProgramId, Long courseId) {
        QueryWrapper<TCUsualScore> tcUsualScoreQueryWrapper = new QueryWrapper<>();
        tcUsualScoreQueryWrapper.lambda().eq(TCUsualScore::getCultureProgramId, cultureProgramId)
                .eq(TCUsualScore::getCourseId, courseId);
        if (tcUsualScoreService.count(tcUsualScoreQueryWrapper) > 0) {
            throw ExceptionResultEnum.ERROR.exception("已导入平时成绩，无法删除");
        }
        QueryWrapper<TCFinalScore> tcFinalScoreQueryWrapper = new QueryWrapper<>();
        tcFinalScoreQueryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId)
                .eq(TCFinalScore::getCourseId, courseId);
        if (tcFinalScoreService.count(tcFinalScoreQueryWrapper) > 0) {
            throw ExceptionResultEnum.ERROR.exception("已导入期末成绩，无法删除");
        }
    }
}
